==> gnotepad+_1.0.2.orig.tar.gz: gnotepad+-1.0.2/src/dialog.c <==
/* vi:set ts=8 sts=0 sw=8:
 * $Id: dialog.c,v 1.5 1998/10/05 15:29:00 kahn Exp kahn $
 *
 * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <string.h>
#include <sys/types.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "main.h"
#include "dialog.h"


/*** local types ***/
typedef struct dlg_data {
	GtkWidget *     dlg;	/* top level dialog widget */
	GtkWidget **    wgts;	/* widgets for dialog buttons */
	short           num;	/* number of buttons */
	short           id;	/* id of button pushed, starts at 1. -1 error */
	GtkSignalFunc **cbs;	/* array of callbacks for button widgets */
	void **         cbd;	/* array of callback data */
} dlg_data_t;


/*** local function prototypes ***/
static void def_cb(GtkWidget *wgt, gpointer cbdata);


/*** global function definitions ***/
/*
 * PUBLIC: do_dialog()
 *
 * generic routine to create a dialog box.
 *
 * PARAMETERS:
 *     title   - title of dialog box.  if NULL, default is "Dialog Title"
 *     msg     - message to user.  if NULL, default is "Hello world!"
 *     num     - number of buttons.
 *     buttons - array of button text.  e.g., { "Yes", "No", "Cancel" }
 *               use GE_BUTTON_OK (et al) for both Gnome and non-Gnome.
 *               see dialog.h for definitions.
 *     dflt    - default button number, starting at 1.  -1 for no default.
 *     cbs     - array of callback functions.  if cbs is NULL, then the
 *               default callback function will be used.
 *     cbd     - array of callback data.  if cbd is NULL, then the default
 *               callback data (dlgdata) is used.
 *     block   - blocking or not.  if blocking, do_dialog() does not return
 *               control back to the caller until the user has pushed one of
 *               the buttons (thus causing the dialog box to be destroyed).
 *               for now, this is always blocking.
 *
 * RETURN VALUES:
 *     blocking: number of button which was pushed by user, starting at 1.
 *     non-blocking: always returns success unless error.
 *     -1 if error.
 */
int
do_dialog(char *title, char *msg, short num, char **buttons, short dflt,
	GtkSignalFunc **cbs, void **cbd, bool_t block)
{
	GtkWidget *tmp;
	GtkWidget **wgts;	/* array to hold all the button widgets */
	GtkWidget *dlg;		/* top level dialog widget */
	dlg_data_t *dlgdata;	/* dialog callback data */
	GtkSignalFunc *cbfunc;	/* callback func */
	char *cbdata;
	short i, ret;

	/* for now, it's always blocking */
	g_assert(block == TRUE);

	if (num < 1 || buttons == NULL)
		return -1;

	dlg = gtk_dialog_new();
	
	gtk_window_set_title(GTK_WINDOW(dlg), (title) ? title : "Dialog Title");

	tmp = gtk_label_new((msg) ? msg : "Hello world!");
	gtk_box_pack_start(
		GTK_BOX(GTK_DIALOG(dlg)->vbox), tmp, TRUE, FALSE, 10);
	gtk_widget_show(tmp);

	wgts = (GtkWidget **)g_malloc(num * sizeof(GtkWidget *));
	dlgdata = (dlg_data_t *)g_malloc0(sizeof(dlg_data_t));
	dlgdata->dlg = dlg;
	dlgdata->wgts = (GtkWidget **)g_malloc0(num * sizeof(GtkWidget *));
	dlgdata->num = num;
	dlgdata->id = -1;
	dlgdata->cbs = cbs;
	dlgdata->cbd = cbd;

	for (i = 0; i < num; i++) {
		tmp = gtk_button_new_with_label(buttons[i]);
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->action_area),
			tmp, TRUE, TRUE, 2);

		dlgdata->wgts[i] = tmp;

		cbfunc = (cbs && cbs[i]) ? cbs[i] : (GtkSignalFunc *)def_cb;
		cbdata = (cbd && cbd[i]) ? (char *)(cbd[i]) : (char *)dlgdata;
		gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
			GTK_SIGNAL_FUNC(cbfunc), cbdata);
		if (cbfunc != (GtkSignalFunc *)def_cb) {
			gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
				GTK_SIGNAL_FUNC(def_cb), dlgdata);
		}

		gtk_widget_show(tmp);

		if (dflt == i + 1) {
			GTK_WIDGET_SET_FLAGS(tmp, GTK_CAN_DEFAULT);
			gtk_widget_grab_default(tmp);
		}
	}

	gtk_widget_show(dlg);
	gtk_grab_add(dlg);

	if (block) {
		/* loop around doing nothing until dialog widget is gone */
		while (dlgdata->dlg != NULL)
			gtk_main_iteration_do(TRUE);

		ret = dlgdata->id + 1;
		g_free(dlgdata->wgts);
		g_free(dlgdata);
		g_free(wgts);
		return ret;
	}

	return 0;	/* non-blocking always returns success */
} /* do_dialog */


/*
 * PUBLIC: do_dialog_yes_no_cancel()
 *
 * wrapper for yes-no-cancel dialog box
 *
 */
int
do_dialog_yes_no_cancel(char *title, char *msg)
{
	char *buttons[] = { BUTTON_YES, BUTTON_NO, BUTTON_CANCEL };

	return do_dialog(title, msg, 3, buttons, 3, NULL, NULL, TRUE);
} /* do_dialog_yes_no_cancel */


/*
 * PUBLIC: do_dialog_ok()
 *
 * wrapper for 'Ok' dialog box
 *
 */
int
do_dialog_ok(char *title, char *msg)
{
	char *buttons[] = { BUTTON_OK };

	return do_dialog(title, msg, 1, buttons, 1, NULL, NULL, TRUE);
} /* do_dialog_ok */


/*** local function definitions ***/
/*
 * PRIVATE: def_cb
 *
 * callback for all button widgets used in do_dialog().  upon invocation,
 * the callback data contains an array of the widget pointers created in
 * do_dialog().  this array is scanned and if a match is found, the id num
 * in the callback data is set, the top-level dialog widget is destroyed,
 * and the dialog widget is set to NULL.  setting the dialog widget to NULL
 * will cause the while loop in do_dialog() to exit.
 */
static void
def_cb(GtkWidget *wgt, gpointer cbdata)
{
	short i;
	dlg_data_t *data = (dlg_data_t *)cbdata;

	g_assert(data != NULL);
	g_assert(data->num > 0);
	g_assert(data->wgts != NULL);
	g_assert(data->id == -1);

	for (i = 0; i < data->num; i++) {
		if (data->wgts[i] == wgt) {
			data->id = i;
			break;
		}
	}

	gtk_widget_destroy(data->dlg);
	data->dlg = NULL;	/* this exits the while loop in do_dialog() */
} /* def_cb */


/* the end */

==> gnotepad+-1.0.8.tar.gz: gnotepad+-1.0.8/src/dialog.c <==
/* vi:set ts=8 sts=0 sw=8:
 * $Id: dialog.c,v 1.7 1999/01/13 21:08:41 kahn Exp kahn $
 *
 * Copyright (C) 1998 Andy C. Kahn <kahn@zk3.dec.com>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <string.h>
#include <sys/types.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "main.h"
#include "misc.h"
#include "dialog.h"


/*** local types ***/
typedef struct dlg_data {
	GtkWidget *     dlg;	/* top level dialog widget */
	GtkWidget **    wgts;	/* widgets for dialog buttons */
	short           num;	/* number of buttons */
	short           id;	/* id of button pushed, starts at 1. -1 error */
	GtkSignalFunc **cbs;	/* array of callbacks for button widgets */
	void **         cbd;	/* array of callback data */
} dlg_data_t;


/*** local function prototypes ***/
static void def_cb(GtkWidget *wgt, gpointer cbdata);


/*** global function definitions ***/
/*
 * PUBLIC: do_dialog()
 *
 * generic routine to create a dialog box.
 *
 * PARAMETERS:
 *     title   - title of dialog box.  if NULL, default is "Dialog Title"
 *     msg     - message to user.  if NULL, default is "Hello world!"
 *     num     - number of buttons.
 *     buttons - array of button text.  e.g., { "Yes", "No", "Cancel" }
 *               use GE_BUTTON_OK (et al) for both Gnome and non-Gnome.
 *               see dialog.h for definitions.
 *     dflt    - default button number, starting at 1.  -1 for no default.
 *     cbs     - array of callback functions.  if cbs is NULL, then the
 *               default callback function will be used.
 *     cbd     - array of callback data.  if cbd is NULL, then the default
 *               callback data (dlgdata) is used.
 *     block   - blocking or not.  if blocking, do_dialog() does not return
 *               control back to the caller until the user has pushed one of
 *               the buttons (thus causing the dialog box to be destroyed).
 *               for now, this is always blocking.
 *
 * RETURN VALUES:
 *     blocking: number of button which was pushed by user, starting at 1.
 *     non-blocking: always returns success unless error.
 *     -1 if error.
 */
int
do_dialog(char *title, char *msg, short num, char **buttons, short dflt,
	GtkSignalFunc **cbs, void **cbd, bool_t block)
{
	GtkWidget *tmp;
	GtkWidget **wgts;	/* array to hold all the button widgets */
	GtkWidget *dlg;		/* top level dialog widget */
	dlg_data_t *dlgdata;	/* dialog callback data */
	GtkSignalFunc *cbfunc;	/* callback func */
	char *cbdata;
	short i, ret;

	/* for now, it's always blocking */
	g_assert(block == TRUE);

	if (num < 1 || buttons == NULL)
		return -1;

	dlg = gtk_dialog_new();
	
	gtk_window_set_title(GTK_WINDOW(dlg), (title) ? title : "Dialog Title");

	tmp = gtk_label_new((msg) ? msg : "Hello world!");
	gtk_box_pack_start(
		GTK_BOX(GTK_DIALOG(dlg)->vbox), tmp, TRUE, FALSE, 10);
	gtk_widget_show(tmp);

	wgts = (GtkWidget **)g_malloc(num * sizeof(GtkWidget *));
	dlgdata = (dlg_data_t *)g_malloc0(sizeof(dlg_data_t));
	dlgdata->dlg = dlg;
	dlgdata->wgts = (GtkWidget **)g_malloc0(num * sizeof(GtkWidget *));
	dlgdata->num = num;
	dlgdata->id = -1;
	dlgdata->cbs = cbs;
	dlgdata->cbd = cbd;

	for (i = 0; i < num; i++) {
		tmp = gtk_button_new_with_label(buttons[i]);
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->action_area),
			tmp, TRUE, TRUE, 2);

		dlgdata->wgts[i] = tmp;

		cbfunc = (cbs && cbs[i]) ? cbs[i] : (GtkSignalFunc *)def_cb;
		cbdata = (cbd && cbd[i]) ? (char *)(cbd[i]) : (char *)dlgdata;
		gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
			GTK_SIGNAL_FUNC(cbfunc), cbdata);
		if (cbfunc != (GtkSignalFunc *)def_cb) {
			gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
				GTK_SIGNAL_FUNC(def_cb), dlgdata);
		}

		gtk_widget_show(tmp);

		if (dflt == i + 1) {
			GTK_WIDGET_SET_FLAGS(tmp, GTK_CAN_DEFAULT);
			gtk_widget_grab_default(tmp);
		}
	}

	gtk_widget_show(dlg);
	gtk_grab_add(dlg);

	if (block) {
		/* loop around doing nothing until dialog widget is gone */
		while (dlgdata->dlg != NULL)
			gtk_main_iteration_do(TRUE);

		ret = dlgdata->id + 1;
		g_free(dlgdata->wgts);
		g_free(dlgdata);
		g_free(wgts);
		return ret;
	}

	return 0;	/* non-blocking always returns success */
} /* do_dialog */


/*
 * PUBLIC: do_dialog_yes_no_cancel()
 *
 * wrapper for yes-no-cancel dialog box
 */
int
do_dialog_yes_no_cancel(char *title, char *msg)
{
	char *buttons[] = { BUTTON_YES, BUTTON_NO, BUTTON_CANCEL };

	return do_dialog(title, msg, 3, buttons, 3, NULL, NULL, TRUE);
} /* do_dialog_yes_no_cancel */


/*
 * PUBLIC: do_dialog_ok()
 *
 * wrapper for 'Ok' dialog box
 */
int
do_dialog_ok(char *title, char *msg)
{
	char *buttons[] = { BUTTON_OK };

	return do_dialog(title, msg, 1, buttons, 1, NULL, NULL, TRUE);
} /* do_dialog_ok */


/*
 * PUBLIC: get_dialog_entry
 *
 * creates a basic, non-modal entry dialog with two buttons: "Ok" and "Cancel".
 * this routine is still a little rough and should be improved (e.g., made more
 * generic), but it does the job for now.
 *
 * in:
 *	title:		dialog box title.  if NULL uses "Entry Dialog Title" as
 *			the default.
 *	labeltxt:	the message to be printed to the user.  if NULL, no
 *			text is added/shown.
 *	dialogtxt:	the text to the left of the entry space.  if NULL, no
 *			text is added/shown.
 *	entry_len:	max length of the entry box/field.
 *	entrytxt:	if non-NULL, the text to be initially placed into the
 *			entry field.
 *	ok_cb:		if non-NULL, callback when "Ok" is clicked.
 *	ok_cbdata:	callback data (can be NULL)
 *	cancel_cb:	if non-NULL, callback when "Cancel" is clicked.
 *	cancel_cbdata:	callback data (can be NULL)
 *	destroy_cb:	if non-NULL, callback for the top level widget's
 *			"destroy" event.
 *	destroy_cbdata:	callback data (can be NULL)
 *
 * out:
 *	toplev:		pointer to top-level dialog widget
 *	entry:		pointer to entry widget
 */
void
get_dialog_entry(char *title, char *labeltxt, char *dialogtxt,
	int entry_len, char *entrytxt,
	GtkSignalFunc ok_cb, gpointer ok_cbdata,
	GtkSignalFunc cancel_cb, gpointer cancel_cbdata,
	GtkSignalFunc destroy_cb, gpointer destroy_cbdata,
	GtkWidget **toplev, GtkWidget **entry)
{
	GtkWidget *vbox, *tmp, *hbox;

	*toplev = gtk_dialog_new();

	gtk_window_set_title(GTK_WINDOW(*toplev),
			     (title) ? title : "Entry Dialog Title");
	if (destroy_cb)
		gtk_signal_connect(GTK_OBJECT(*toplev), "destroy",
				   GTK_SIGNAL_FUNC(destroy_cb), destroy_cbdata);

	gtk_container_border_width(GTK_CONTAINER(*toplev), 6);
	vbox = GTK_DIALOG(*toplev)->vbox;

	if (labeltxt) {
		tmp = gtk_label_new(labeltxt);
		gtk_box_pack_start(GTK_BOX(vbox), tmp, TRUE, FALSE, 5);
	}

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 5);

	if (dialogtxt) {
		tmp = gtk_label_new(dialogtxt);
		gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, TRUE, 5);
	}

	*entry = gtk_entry_new_with_max_length(entry_len);
	if (entrytxt)
		gtk_entry_set_text(GTK_ENTRY(*entry), entrytxt);
	gtk_box_pack_start(GTK_BOX(hbox), *entry, FALSE, TRUE, 10);

	hbox = GTK_DIALOG(*toplev)->action_area;
	tmp = misc_button_new_w_label("Ok", ok_cb, ok_cbdata, hbox,
#ifdef GTK_HAVE_FEATURES_1_1_0
				      GTK_RELIEF_NORMAL,
#endif
				      NULL, 0, 0, 0, 0);

	tmp = misc_button_new_w_label("Cancel", cancel_cb, cancel_cbdata, hbox,
#ifdef GTK_HAVE_FEATURES_1_1_0
				      GTK_RELIEF_NORMAL,
#endif
				      NULL, 0, 0, 0, 0);

	gtk_widget_show_all(*toplev);
} /* get_dialog_entry */


/*** local function definitions ***/
/*
 * PRIVATE: def_cb
 *
 * callback for all button widgets used in do_dialog().  upon invocation,
 * the callback data contains an array of the widget pointers created in
 * do_dialog().  this array is scanned and if a match is found, the id num
 * in the callback data is set, the top-level dialog widget is destroyed,
 * and the dialog widget is set to NULL.  setting the dialog widget to NULL
 * will cause the while loop in do_dialog() to exit.
 */
static void
def_cb(GtkWidget *wgt, gpointer cbdata)
{
	short i;
	dlg_data_t *data = (dlg_data_t *)cbdata;

	g_assert(data != NULL);
	g_assert(data->num > 0);
	g_assert(data->wgts != NULL);
	g_assert(data->id == -1);

	for (i = 0; i < data->num; i++) {
		if (data->wgts[i] == wgt) {
			data->id = i;
			break;
		}
	}

	gtk_widget_destroy(data->dlg);
	data->dlg = NULL;	/* this exits the while loop in do_dialog() */
} /* def_cb */


/* the end */
